#version 300 es

precision highp float;

uniform float u_Time;
uniform vec2 u_Resolution;
uniform sampler2D u_Texture;

in vec2 v_TexCoord;

layout(location = 0) out vec4 o_FragColor;

#define GOLDEN_ANGLE 2.39996

#define ITERATIONS 50

mat2 rot = mat2(cos(GOLDEN_ANGLE), sin(GOLDEN_ANGLE), -sin(GOLDEN_ANGLE), cos(GOLDEN_ANGLE));

//-------------------------------------------------------------------------------------------
vec3 Bokeh(sampler2D tex, vec2 uv, float radius)
{
    vec3 acc = vec3(0), div = acc;
    float r = 1.;
    vec2 vangle = vec2(0.0,radius*.01 / sqrt(float(ITERATIONS)));

    for (int j = 0; j < ITERATIONS; j++)
    {
        // the approx increase in the scale of sqrt(0, 1, 2, 3...)
        r += 1. / r;
        vangle = rot * vangle;
        vec3 col = texture(tex, uv + (r-1.) * vangle).xyz; /// ... Sample the image
        col = col * col *1.8; // ... Contrast it for better highlights - leave this out elsewhere.
        vec3 bokeh = pow(col, vec3(4));
        acc += col * bokeh;
        div += bokeh;
    }
    return acc / div;
}


void main() {
    vec2 uv = gl_FragCoord.xy / u_Resolution; //... with correct aspect ratio

    float time = mod(u_Time*.2 +.25, 3.0);

    float rad = .8 - .8*cos(time * 6.283);

    o_FragColor = vec4(Bokeh(u_Texture, uv, rad), 1.0);
}
