#version 300 es

precision highp float;

uniform float u_Time;
uniform vec2 u_Resolution;
uniform sampler2D u_Texture; // clamp
uniform sampler2D u_NoiseTexture;

in vec2 v_TexCoord;

layout(location = 0) out vec4 o_FragColor;

float rand(float n){return fract(sin(n) * 43758.5453123);}

float noise(float p){
    float fl = floor(p);
    float fc = fract(p);
    return mix(rand(fl), rand(fl + 1.0), fc);
}

float blockyNoise(vec2 uv, float threshold, float scale, float seed)
{
    float scroll = floor(u_Time + sin(11.0 *  u_Time) + sin(u_Time) ) * 0.77;
    vec2 noiseUV = uv.yy / scale + scroll;
    float noise2 = texture(u_NoiseTexture, noiseUV).r;

    float id = floor( noise2 * 20.0);
    id = noise(id + seed) - 0.5;


    if ( abs(id) > threshold ) {
        id = 0.0;
    }

    return id;
}

void main() {
    float rgbIntesnsity = 0.1 + 0.1 * sin(u_Time* 3.7);
    float displaceIntesnsity = 0.2 +  0.3 * pow( sin(u_Time * 1.2), 5.0);
    float interlaceIntesnsity = 0.01;
    float dropoutIntensity = 0.1;


    vec2 uv = gl_FragCoord.xy/u_Resolution.xy;

    float displace = blockyNoise(uv + vec2(uv.y, 0.0), displaceIntesnsity, 25.0, 66.6);
    displace *= blockyNoise(uv.yx + vec2(0.0, uv.x), displaceIntesnsity, 111.0, 13.7);

    uv.x += displace ;

    vec2 offs = 0.1 * vec2(blockyNoise(uv.xy + vec2(uv.y, 0.0), rgbIntesnsity, 65.0, 341.0), 0.0);

    float colr = texture(u_Texture, uv-offs).r;
    float colg = texture(u_Texture, uv).g;
    float colb = texture(u_Texture, uv +offs).b;


    float line = fract(gl_FragCoord.y / 3.0);
    vec3 mask = vec3(3.0, 0.0, 0.0);

    if (line > 0.333) {
        mask = vec3(0.0, 3.0, 0.0);
    }

    if (line > 0.666) {
        mask = vec3(0.0, 0.0, 3.0);
    }


    float maskNoise = blockyNoise(uv, interlaceIntesnsity, 90.0, u_Time) * max(displace, offs.x);

    maskNoise = 1.0 - maskNoise;

    if ( maskNoise == 1.0) {
        mask = vec3(1.0);
    }

    float dropout = blockyNoise(uv, dropoutIntensity, 11.0, u_Time) * blockyNoise(uv.yx, dropoutIntensity, 90.0, u_Time);
    mask *= (1.0 - 5.0 * dropout);


    o_FragColor = vec4(mask * vec3(colr, colg, colb), 1.0);
}
