#version 300 es

precision highp float;

uniform float u_Time;
uniform vec2 u_Resolution;
uniform sampler2D u_Texture; // repeat
uniform sampler2D u_NoiseTexture0;
uniform sampler2D u_NoiseTexture1;

in vec2 v_TexCoord;

layout(location = 0) out vec4 o_FragColor;

#define pi 3.1415926

vec3 colorSplit(vec2 uv, vec2 s)
{
    vec3 color;
    color.r = texture(u_Texture, uv - s).r;
    color.g = texture(u_Texture, uv    ).g;
    color.b = texture(u_Texture, uv + s).b;
    return color;
}

vec2 interlace(vec2 uv, float s)
{
    uv.x += s * (4.0 * fract((uv.y * u_Resolution.y) / 2.0) - 1.0);
    return uv;
}

vec2 fault(vec2 uv, float s)
{
    //float v = (0.5 + 0.5 * cos(2.0 * pi * uv.y)) * (2.0 * uv.y - 1.0);
    float v = pow(0.5 - 0.5 * cos(2.0 * pi * uv.y), 100.0) * sin(2.0 * pi * uv.y);
    uv.x += v * s;
    return uv;
}

vec2 rnd(vec2 uv, float s)
{
    uv.x += s * (2.0 * texture(u_NoiseTexture0, uv * 0.05).x - 1.0);
    return uv;
}

void main() {
    float t = u_Time / 10.0;

    vec2 uv = gl_FragCoord.xy / u_Resolution.xy;

    //float s = pow(0.5 + 0.5 * cos(2.0 * pi * t), 1000.0);
    float s = texture(u_NoiseTexture0, vec2(t * 0.2, 0.5)).r;

    uv = interlace(uv, s * 0.005);
    //uv = fault(uv, s);
    float r = texture(u_NoiseTexture1, vec2(t, 0.0)).x;
    //uv = fault(uv + vec2(0.0, fract(t * 20.0)), r) - vec2(0.0, fract(t * 20.0));
    uv = fault(uv + vec2(0.0, fract(t * 2.0)), 5.0 * sign(r) * pow(abs(r), 5.0)) - vec2(0.0, fract(t * 2.0));
    uv = rnd(uv, s * 0.02);

    vec3 color = colorSplit(uv, vec2(s * 0.02, 0.0));
    //vec2 m = texture(u_NoiseTexture1, uv).xy;
    color = mix(color, texture(u_NoiseTexture0, 0.5 * uv + t * 100.0).rgb, 0.25);

    o_FragColor = vec4(color, 1.0);
}
