#version 300 es

precision highp float;

uniform float u_Time;
uniform vec2 u_Resolution;
uniform sampler2D u_Texture; // clamp
uniform sampler2D u_NoiseTexture;

in vec2 v_TexCoord;

layout(location = 0) out vec4 o_FragColor;

void main() {
    vec2 uv = gl_FragCoord.xy / u_Resolution.xy;
    vec2 block = floor(gl_FragCoord.xy / vec2(16));
    vec2 uv_noise = block / vec2(64);
    uv_noise += floor(vec2(u_Time) * vec2(1234.0, 3543.0)) / vec2(64);

    float block_thresh = pow(fract(u_Time * 1236.0453), 2.0) * 0.2;
    float line_thresh = pow(fract(u_Time * 2236.0453), 3.0) * 0.7;

    vec2 uv_r = uv, uv_g = uv, uv_b = uv;

    // glitch some blocks and lines
    if (texture(u_NoiseTexture, uv_noise).r < block_thresh || texture(u_NoiseTexture, vec2(uv_noise.y, 0.0)).g < line_thresh) {

        vec2 dist = (fract(uv_noise) - 0.5) * 0.3;
        uv_r += dist * 0.1;
        uv_g += dist * 0.2;
        uv_b += dist * 0.125;
    }

    o_FragColor.r = texture(u_Texture, uv_r).r;
    o_FragColor.g = texture(u_Texture, uv_g).g;
    o_FragColor.b = texture(u_Texture, uv_b).b;
    o_FragColor.a = 1.0;

    // loose luma for some blocks
    if (texture(u_NoiseTexture, uv_noise).g < block_thresh) {
        o_FragColor.rgb = o_FragColor.ggg;
    }

    // discolor block lines
    if (texture(u_NoiseTexture, vec2(uv_noise.y, 0.0)).b * 3.5 < line_thresh) {
        o_FragColor.rgb = vec3(0.0, dot(o_FragColor.rgb, vec3(1.0)), 0.0);
    }

    // interleave lines in some blocks
    if (texture(u_NoiseTexture, uv_noise).g * 1.5 < block_thresh || texture(u_NoiseTexture, vec2(uv_noise.y, 0.0)).g * 2.5 < line_thresh) {
        float line = fract(gl_FragCoord.y / 3.0);

        vec3 mask = vec3(3.0, 0.0, 0.0);

        if (line > 0.333) {
            mask = vec3(0.0, 3.0, 0.0);
        }

        if (line > 0.666) {
            mask = vec3(0.0, 0.0, 3.0);
        }

        o_FragColor.xyz *= mask;
    }
}
