#version 300 es

precision highp float;

uniform float u_Time;
uniform vec2 u_Resolution;
uniform sampler2D u_NoiseTexture;

layout(location = 0) out vec4 o_FragColor;

const float M_PI_F = 3.14159;

vec2 noiseOffset(float time, vec2 uv, vec3 angleParamsX, vec3 angleParamsY, vec2 angleParamsTime, float baseDistance, vec3 distanceParamsX, vec3 distanceParamsY) {
    float angle = M_PI_F * (sin(angleParamsX.x * uv.x + angleParamsX.y * time + angleParamsX.z) + sin(angleParamsY.x * uv.y + angleParamsY.y * time + angleParamsY.z) + sin(angleParamsTime.x * time + angleParamsTime.y));
    float distance = baseDistance + baseDistance * (sin(distanceParamsX.x * uv.x + distanceParamsX.y * time + distanceParamsX.z) + sin(distanceParamsY.x * uv.y + distanceParamsY.y * time + distanceParamsY.z));
    return vec2(cos(angle), sin(angle)) * distance;
}

void main() {
    vec2 centeredCoords = ((gl_FragCoord.xy / u_Resolution.xy) - 0.5) * 1.2;
    centeredCoords.y *= u_Resolution.y / u_Resolution.x;
    float centerDistance = length(centeredCoords);
    vec2 noiseUV1 = centeredCoords;
    noiseUV1 += noiseOffset(u_Time, noiseUV1, vec3(2.3, 0.51, 0.0), vec3(-1.9, 0.47, 1.0), vec2(0.4, 0.4), 0.02, vec3(3.1, 0.44, 1.3), vec3(3.4, 0.53, 0.0));
    noiseUV1 *= (1.0 - 0.3 * pow(centerDistance * 2.0, 1.2));
    noiseUV1.y -= u_Time * 0.05;
    noiseUV1.x += 0.3 * sin(u_Time * 0.13);
    noiseUV1 *= 0.7; // scale it so we get larger specks

    vec2 noiseUV2 = centeredCoords;
    noiseUV2 += noiseOffset(u_Time, noiseUV2, vec3(-1.62, 0.3, 2.1), vec3(-1.83, 0.42, 0.3), vec2(-0.2, 0.19), 0.01, vec3(4.28, 0.23, 0.5), vec3(3.93, 0.19, 1.1));
    noiseUV2 *= (1.0 - 0.4 * pow(centerDistance * 2.0, 1.3));
    noiseUV2.x += 0.2 * sin(u_Time * 0.22 + 0.5);
    noiseUV2.y -= u_Time * 0.02;
    noiseUV2 *= 0.83;

    float edgeAttenuation = pow(centerDistance, 2.0);

    float visibilitySample1 = texture(u_NoiseTexture, noiseUV1).r;
    float colorSample1 = texture(u_NoiseTexture, vec2(noiseUV1.x + u_Time * 0.005, noiseUV1.y)).g;
    float visibility1 = smoothstep(0.8, 0.9, visibilitySample1 - edgeAttenuation * 0.5);

    float visibilitySample2 = texture(u_NoiseTexture, noiseUV2).b;
    float colorSample2 = texture(u_NoiseTexture, vec2(noiseUV2.x - u_Time * 0.009, noiseUV2.y)).r;
    float visibility2 = smoothstep(0.85, 0.9, visibilitySample2 - edgeAttenuation * 0.4);

    o_FragColor = vec4(mix(vec3(1.0,0.2,0.9), vec3(0.1,0.4,1.0), float(smoothstep(0.0, 1.0, colorSample2))) * visibility2 + mix(vec3(1.0, 0.1, 0.6), vec3(0.2, 0.8, 1.0), float(smoothstep(0.0, 1.0, colorSample1))) * visibility1, 1.0);
}
