#version 300 es

precision highp float;

uniform float u_Time;
uniform vec2 u_Resolution;
uniform sampler2D u_Texture; // clamp
uniform sampler2D u_NoiseTexture;

in vec2 v_TexCoord;

layout(location = 0) out vec4 o_FragColor;

float noise(vec2 p)
{
    float s = texture(u_NoiseTexture,vec2(1.,2.*cos(u_Time))*u_Time*8. + p*1.).x;
    s *= s;
    return s;
}

float onOff(float a, float b, float c)
{
    return step(c, sin(u_Time + a*cos(u_Time*b)));
}

float ramp(float y, float start, float end)
{
    float inside = step(start,y) - step(end,y);
    float fact = (y-start)/(end-start)*inside;
    return (1.-fact) * inside;

}

float stripes(vec2 uv)
{

    float noi = noise(uv*vec2(0.5,1.) + vec2(1.,3.));
    return ramp(mod(uv.y*4. + u_Time/2.+sin(u_Time + sin(u_Time*0.63)),1.),0.5,0.6)*noi;
}

vec3 getVideo(vec2 uv)
{
    vec2 look = uv;
    float window = 1./(1.+20.*(look.y-mod(u_Time/4.,1.))*(look.y-mod(u_Time/4.,1.)));
    look.x = look.x + sin(look.y*10. + u_Time)/50.*onOff(4.,4.,.3)*(1.+cos(u_Time*80.))*window;
    float vShift = 0.4*onOff(2.,3.,.9)*(sin(u_Time)*sin(u_Time*20.) +
    (0.5 + 0.1*sin(u_Time*200.)*cos(u_Time)));
    look.y = mod(look.y + vShift, 1.);
    vec3 video = vec3(texture(u_Texture,look));
    return video;
}

vec2 screenDistort(vec2 uv)
{
    uv -= vec2(.5,.5);
    uv = uv*1.2*(1./1.2+2.*uv.x*uv.x*uv.y*uv.y);
    uv += vec2(.5,.5);
    return uv;
}

void main() {
    vec2 uv = gl_FragCoord.xy / u_Resolution.xy;
    uv = screenDistort(uv);
    vec3 video = getVideo(uv);
    float vigAmt = 3.+.3*sin(u_Time + 5.*cos(u_Time*5.));
    float vignette = (1.-vigAmt*(uv.y-.5)*(uv.y-.5))*(1.-vigAmt*(uv.x-.5)*(uv.x-.5));

    video += stripes(uv);
    video += noise(uv*2.)/2.;
    video *= vignette;
    video *= (12.+mod(uv.y*30.+u_Time,1.))/13.;

    o_FragColor = vec4(video,1.0);
}
