#version 300 es

precision highp float;

uniform float u_Time;
uniform vec2 u_Resolution;
uniform sampler2D u_Texture; // clamp
uniform sampler2D u_NoiseTexture;

in vec2 v_TexCoord;

layout(location = 0) out vec4 o_FragColor;

const float range = 0.05;
const float noiseQuality = 250.0;
const float noiseIntensity = 0.0088;
const float offsetIntensity = 0.02;
const float colorOffsetIntensity = 1.3;

float rand(vec2 co)
{
    return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453);
}

float verticalBar(float pos, float uvY, float offset)
{
    float edge0 = (pos - range);
    float edge1 = (pos + range);

    float x = smoothstep(edge0, pos, uvY) * offset;
    x -= smoothstep(pos, edge1, uvY) * offset;
    return x;
}

void main() {
    vec2 uv = gl_FragCoord.xy / u_Resolution;

    for (float i = 0.0; i < 0.71; i += 0.1313)
    {
        float d = mod(u_Time * i, 1.7);
        float o = sin(1.0 - tan(u_Time * 0.24 * i));
        o *= offsetIntensity;
        uv.x += verticalBar(d, uv.y, o);
    }

    float uvY = uv.y;
    uvY *= noiseQuality;
    uvY = float(int(uvY)) * (1.0 / noiseQuality);
    float noise = rand(vec2(u_Time * 0.00001, uvY));
    uv.x += noise * noiseIntensity;

    vec2 offsetR = vec2(0.006 * sin(u_Time), 0.0) * colorOffsetIntensity;
    vec2 offsetG = vec2(0.0073 * (cos(u_Time * 0.97)), 0.0) * colorOffsetIntensity;

    float r = texture(u_Texture, uv + offsetR).r;
    float g = texture(u_Texture, uv + offsetG).g;
    float b = texture(u_Texture, uv).b;

    o_FragColor = vec4(r, g, b, 1.0);
}
